package tech.riemann.ims.service.acl.impl;

import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import javax.annotation.Nonnull;

import org.nutz.dao.Dao;
import org.nutz.lang.Lang;
import org.nutz.lang.Strings;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;

import club.zhcs.lina.utils.tree.Tree;
import club.zhcs.lina.utils.tree.Treeable;
import lombok.RequiredArgsConstructor;
import tech.riemann.ims.dto.response.PermissionTree;
import tech.riemann.ims.entity.acl.Permission;
import tech.riemann.ims.mapper.acl.PermissionMapper;
import tech.riemann.ims.service.acl.IPermissionService;

/**
 * <p>
 * 权限 服务实现类
 * </p>
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-09-10 18:07:15
 */
@Service
@RequiredArgsConstructor
public class PermissionServiceImpl extends ServiceImpl<PermissionMapper, Permission> implements IPermissionService {

    private final Dao dao;

    LoadingCache<String, List<Permission>> cache = CacheBuilder.newBuilder()
                                                               .concurrencyLevel(8)
                                                               .expireAfterAccess(10, TimeUnit.MINUTES)
                                                               .initialCapacity(10)
                                                               .maximumSize(300)
                                                               .recordStats()
                                                               .build(new CacheLoader<String, List<Permission>>() {

                                                                   @Override
                                                                   public List<Permission> load(@Nonnull String key) throws Exception {
                                                                       return list();
                                                                   }
                                                               });

    /**
     * @return
     * @see org.nutz.spring.boot.service.ExtService#dao()
     */
    @Override
    public Dao dao() {
        return dao;
    }

    /**
     * @return
     * @see org.nutz.spring.boot.service.interfaces.EntityService#getEntityType()
     */
    @Override
    public Class<Permission> getEntityType() {
        return Permission.class;
    }

    /**
     * @param force
     * @return
     * @see tech.riemann.laacam.service.acl.IPermissionService#permissionTree(java.lang.Boolean)
     */
    @Override
    public List<Tree<String>> permissionTree(Boolean force) {
        List<Permission> permissions = permissions(force);
        List<Treeable<String>> permissionTrees = permissions.stream()
                                                            .map(PermissionTree::build)
                                                            .collect(Collectors.toList());
        return Tree.from(permissionTrees,
                         permissions.stream()
                                    .filter(organization -> Strings.isBlank(organization.getParentKey()))
                                    .map(Permission::getKey)
                                    .toList());
    }

    /**
     * @param force
     * @return
     * @see tech.riemann.laacam.service.acl.IPermissionService#permissions(java.lang.Boolean)
     */
    @Override
    public List<Permission> permissions(Boolean force) {
        try {
            if (Boolean.TRUE.equals(force)) {
                cache.refresh("all");
            }
            return cache.get("all");
        }
        catch (ExecutionException e) {
            throw Lang.wrapThrow(e);
        }
    }

    /**
     * @param permissions
     * @see tech.riemann.laacam.service.acl.IPermissionService#batchAddPermissions(java.util.List)
     */
    @Override
    public void batchAddPermissions(List<Permission> permissions) {
        clear();
        insert(permissions);
    }

    /**
     * @param permissions
     * @see tech.riemann.laacam.service.acl.IPermissionService#updatePermissions(java.util.List)
     */
    @Override
    public void updatePermissions(List<Permission> permissions) {
        permissions.stream().forEach(permission -> {
            if (exists(Wrappers.<Permission> lambdaQuery().eq(Permission::getKeyPath, permission.getKeyPath()))) {
                update(
                       Wrappers.<Permission> lambdaUpdate()
                               .eq(Permission::getKeyPath, permission.getKeyPath())
                               .set(Permission::getName, permission.getName())
                               .set(Permission::getDescription, permission.getDescription())
                               .set(Permission::getType, permission.getType()));
            } else {
                save(permission);
            }
        });

    }
}
