package tech.riemann.ims.service.acl.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.nutz.dao.Dao;
import org.nutz.lang.Lang;
import org.nutz.lang.Strings;
import org.nutz.lang.random.R;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import club.zhcs.lina.auth.encode.PasswordUtils;
import club.zhcs.lina.auth.jwt.JWTEncoder;
import club.zhcs.lina.auth.service.AuthUser;
import club.zhcs.lina.starter.exception.BizException;
import lombok.RequiredArgsConstructor;
import tech.riemann.ims.controller.platform.auth.AuthController.LoginDTO;
import tech.riemann.ims.dto.response.PermissionInfo;
import tech.riemann.ims.dto.response.RoleInfo;
import tech.riemann.ims.entity.acl.Permission;
import tech.riemann.ims.entity.acl.Role;
import tech.riemann.ims.entity.acl.User;
import tech.riemann.ims.entity.acl.UserPermission;
import tech.riemann.ims.entity.acl.UserRole;
import tech.riemann.ims.mapper.acl.UserMapper;
import tech.riemann.ims.mapper.acl.UserPermissionMapper;
import tech.riemann.ims.mapper.acl.UserRoleMapper;
import tech.riemann.ims.service.acl.IUserService;

/**
 * <p>
 * 用户 服务实现类
 * </p>
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-09-10 18:07:15
 */
@Service
@RequiredArgsConstructor
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    private final Dao dao;
    private final UserPermissionMapper userPermissionMapper;
    private final UserRoleMapper userRoleMapper;
    private final JWTEncoder jwtEncoder;

    /**
     * @return
     * @see org.nutz.spring.boot.service.ExtService#dao()
     */
    @Override
    public Dao dao() {
        return dao;
    }

    /**
     * @return
     * @see org.nutz.spring.boot.service.interfaces.EntityService#getEntityType()
     */
    @Override
    public Class<User> getEntityType() {
        return User.class;
    }

    /**
     * @param name
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#resetPassword(java.lang.String)
     */
    @Override
    public String resetPassword(String name) {
        String newPassword = R.sg(10).next();

        if (!update(Wrappers.<User> lambdaUpdate()
                            .eq(User::getName, name)
                            .set(User::getPassword,
                                 PasswordUtils.randomSaltEncode(newPassword)))) {
            throw BizException.create("重置密码失败!");
        }
        return newPassword;
    }

    /**
     * @param name
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#permissionsByUserName(java.lang.String)
     */
    @Override
    public List<Permission> permissionsByUserName(String name) {
        return permissionInfosByUserName(name)
                                              .stream()
                                              .filter(PermissionInfo::isSelected)
                                              .collect(Collectors.toList());
    }

    /**
     * @param name
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#permissionInfosByUserName(java.lang.String)
     */
    @Override
    public List<PermissionInfo> permissionInfosByUserName(String name) {
        List<PermissionInfo> directPermissionInfos = directPermissionInfosByUserName(name);
        List<PermissionInfo> indirectPermissionInfos = indirectPermissionInfosByUserName(name);
        directPermissionInfos.stream().forEach(direct -> {
            boolean selected = indirectPermissionInfos.stream()
                                                      .anyMatch(indirect -> Strings.equals(indirect.getKeyPath(), direct.getKeyPath())
                                                                            && indirect.isSelected());
            if (selected) {
                direct.setSelected(selected);
            }
        });
        return directPermissionInfos;
    }

    /**
     * @param name
     * @return
     */
    private List<PermissionInfo> indirectPermissionInfosByUserName(String name) {
        return getBaseMapper().indirectPermissionInfosByUserName(name);
    }

    /**
     * @param name
     * @return
     */
    private List<PermissionInfo> directPermissionInfosByUserName(String name) {
        return getBaseMapper().directPermissionInfosByUserName(name);
    }

    /**
     * @param name
     * @param permissions
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#grant(java.lang.String,
     *      java.util.List)
     */
    @Override
    public boolean grant(String name, List<String> permissions) {
        userPermissionMapper.delete(Wrappers.<UserPermission> lambdaQuery().eq(UserPermission::getUserName, name));
        userPermissionMapper.insert(permissions.stream()
                                               .map(p -> UserPermission.builder().permissionKeyPath(p).userName(name).build())
                                               .collect(Collectors.toList()));
        return true;
    }

    /**
     * @param name
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#roleInfosByUserName(java.lang.String)
     */
    @Override
    public List<RoleInfo> roleInfosByUserName(String name) {
        return getBaseMapper().roleInfosByUserName(name);
    }

    /**
     * @param name
     * @param roles
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#grantRole(java.lang.String,
     *      java.util.List)
     */
    @Override
    public boolean grantRole(String name, List<String> roles) {
        userRoleMapper.delete(Wrappers.<UserRole> lambdaQuery().eq(UserRole::getUserName, name));
        userRoleMapper.insert(roles.stream()
                                   .map(role -> UserRole.builder()
                                                        .userName(name)
                                                        .roleKey(role)
                                                        .build())
                                   .collect(Collectors.toList()));
        return true;
    }

    /**
     * @param login
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#login(tech.riemann.laacam.controller.platform.auth.AuthController.LoginDTO)
     */
    @Override
    public AuthUser login(LoginDTO login) {
        AuthUser authUser = null;
        if (login.getType() == LoginDTO.Type.ACCOUNT) {
            User user = getBaseMapper().selectOne(Wrappers.<User> lambdaQuery().eq(User::getName, login.getUserName()));
            if (Lang.isEmpty(user)) {
                throw Lang.makeThrow(BizException.class, "不存在的用户: %s", login.getUserName());
            }
            if (!PasswordUtils.randomSaltVerify(login.getPassword(), user.getPassword())) {
                throw Lang.makeThrow(BizException.class, "用户: %s 密码不正确", login.getUserName());
            }
            authUser = authUser(user);
        }
        return authUser;
    }

    /**
     * 转换用户
     * 
     * @param user
     * @return
     */
    protected AuthUser authUser(User user) {
        return user == null ? null
                            : AuthUser.builder()
                                      .email(user.getEmail())
                                      .fullName(user.getFullName())
                                      .mobile(user.getEmail())
                                      .password(user.getPassword())
                                      .sex(user.getSex())
                                      .token(jwtEncoder.token(user.getName()))
                                      .refreshToken(jwtEncoder.refreshToken(user.getName()))
                                      .userName(user.getName())
                                      .roles(roleInfosByUserName(user.getName()).stream()
                                                                                .filter(RoleInfo::isSelected)
                                                                                .map(Role::getKey)
                                                                                .toList())
                                      .permissions(permissionInfosByUserName(user.getName()).stream()
                                                                                            .filter(PermissionInfo::isSelected)
                                                                                            .map(Permission::getKeyPath)
                                                                                            .toList())
                                      .build();
    }

    /**
     * @param name
     * @return
     * @see tech.riemann.laacam.service.acl.IUserService#authUser(java.lang.String)
     */
    @Override
    public AuthUser authUser(String name) {
        User user = getBaseMapper().selectOne(Wrappers.<User> lambdaQuery().eq(User::getName, name));
        if (Lang.isEmpty(user)) {
            throw Lang.makeThrow(BizException.class, "不存在的用户: %s", name);
        }
        return authUser(user);
    }
}
